#include <ESP8266WiFi.h>
#include <WiFiClient.h>
#include <ESP8266WebServer.h>
#include "websites.h"

// Source: http://www.esp8266.com/viewtopic.php?f=32&t=3780# - Tom torx@esp8266.com
// Thank you very much Tom
// Incorporated improvement suggested by User treii28@esp8266.com
// Used WiFi-setup as posted at 
// https://github.com/esp8266/Arduino/blob/master/libraries/ESP8266WiFi/examples/WiFiWebServer/WiFiWebServer.ino

// Modyfied by Daniel Suesstrunk, local_dani_21@wunderwald.ch, 2017

const char *ssid = "w";
const char *password = "password";

ESP8266WebServer server(80);

void handleNotFound();
bool loadFromFlash(String path);

void setup()
{
  // Enable serial port for debugging purposes
  Serial.begin ( 115200 );
  Serial.println("ESP starting up...");
 
  WiFi.begin ( ssid, password );
  Serial.println ( "" );

  // Wait for connection
  while ( WiFi.status() != WL_CONNECTED ) {
    delay ( 500 );
    Serial.print ( "." );
  }

  Serial.println ( "" );
  Serial.print ( "Connected to " );
  Serial.println ( ssid );
  Serial.print ( "IP address: " );
  Serial.println ( WiFi.localIP() );

  // in the notFound we search for files ourselves
  server.onNotFound(handleNotFound);
  server.begin();
}

bool loadFromFlash(String path) {
  if(path.endsWith("/")) path += "index.html";
  Serial.println(path);

  int NumFiles = sizeof(files)/sizeof(struct t_websitefiles);
 
  for(int i=0; i<NumFiles; i++) {
    if(path.endsWith(String(files[i].filename))) {     
      _FLASH_ARRAY<uint8_t>* filecontent;
      String dataType = "text/plain";
      unsigned int len = 0;
      WiFiClient client = server.client();
     
      dataType = files[i].mime;
      len = files[i].len;
     
      server.setContentLength(len);
      server.send(200, files[i].mime, "");
     
      filecontent = (_FLASH_ARRAY<uint8_t>*)files[i].content;
     
      filecontent->open();
     
      client.write(*filecontent, 100);
      return true;
    }
  }
 
  return false;
}

void handleNotFound() {
 
  // try to find the file in the flash
  if(loadFromFlash(server.uri())) return;
 
  String message = "File Not Found\n\n";
  message += "URI..........: ";
  message += server.uri();
  message += "\nMethod.....: ";
  message += (server.method() == HTTP_GET)?"GET":"POST";
  message += "\nArguments..: ";
  message += server.args();
  message += "\n";
  for (uint8_t i=0; i<server.args(); i++){
    message += " " + server.argName(i) + ": " + server.arg(i) + "\n";
  }
  message += "\n";
  message += "FreeHeap.....: " + String(ESP.getFreeHeap()) + "\n";
  message += "ChipID.......: " + String(ESP.getChipId()) + "\n";
  message += "FlashChipId..: " + String(ESP.getFlashChipId()) + "\n";
  message += "FlashChipSize: " + String(ESP.getFlashChipSize()) + " bytes\n";
  message += "getCycleCount: " + String(ESP.getCycleCount()) + " Cycles\n";
  message += "Milliseconds.: " + String(millis()) + " Milliseconds\n";
  server.send(404, "text/plain", message);
}

void loop()
{
  // process webserver
  server.handleClient();

}


